<?php
// wcf imports
require_once(WCF_DIR.'lib/acp/package/plugin/AbstractXMLPackageInstallationPlugin.class.php');

/**
 * This PIP checks the requirements
 *
 * @author	Tim Düsterhus
 * @copyright	2011 wbb3addons.de
 * @license	GNU Lesser General Public License <http://www.gnu.org/licenses/lgpl.html>
 * @package	de.wbb3addons.pip.checkRequirements
 */
class RequirementsPackageInstallationPlugin extends AbstractXMLPackageInstallationPlugin {
	/**
	 * Tag in package.xml
	 *
	 * @var string
	 */
	public $tagName = 'requirements';

	/**
	 * The test-results
	 *
	 * @var array<array<integer>
	 */
	public $results = array();

	/**
	 * The lowest result that was reached
	 *
	 * @var integer
	 */
	public $overallStatus = 2;

	/**
	 * The current block and child, to access it via eventlistener
	 *
	 * @var array<mixed>
	 */
	public $block = array(), $child = array();

	/**
	 * Counter for the failed checks
	 *
	 * @var integer
	 */
	public $failed = 0;

	/**
	 * @see PackageInstallationPlugin::install()
	 */
	public function install() {
		parent::install();

		if (!$xml = $this->getXML()) {
			return;
		}

		$requirementsXML = $xml->getElementTree('requirements');

		foreach ($requirementsXML['children'] as $key => $this->block) {
			switch ($this->block['name']) {
				// php checks
				case 'php':
					foreach ($this->block['children'] as $this->child) {
						try {
							if ($this->child['name'] == 'versions') {
								$installedVersion = PHP_VERSION;
								$status = 0;
								$hasRecommend = false;
								foreach ($this->child['children'] as $version) {
									if ($version['attrs']['type'] == 'recommend') $hasRecommend = true;
									if (version_compare($installedVersion, $version['cdata']) >= 0) {
										if ($version['attrs']['type'] == 'minimum' && $status < 1) $status = 1;
										else if ($version['attrs']['type'] == 'recommend' && $status < 2) $status = 2;
									}
								}

								// if there is only a minimum, handle it like recommend
								if (!$hasRecommend && $status == 1) $status = 2;
								$this->addResult('php.version', array('status' => $status, 'version' => $installedVersion));
							}
							else if ($this->child['name'] == 'functions') {
								foreach ($this->child['children'] as $function) {
									if (function_exists($function['cdata'])) $status = 2;
									else {
										if ($function['attrs']['type'] == 'minimum') $status = 0;
										else $status = 1;
									}
									$this->addResult('php.function', array('status' => $status, 'function' => $function['cdata']));
								}
							}
							else if ($this->child['name'] == 'classes') {
								foreach ($this->child['children'] as $class) {
									if (class_exists($class['cdata'], false)) $status = 2;
									else {
										if ($class['attrs']['type'] == 'minimum') $status = 0;
										else $status = 1;
									}
									$this->addResult('php.class', array('status' => $status, 'class' => $class['cdata']));
								}
							}
							else if ($this->child['name'] == 'extensions') {
								foreach ($this->child['children'] as $extension) {
									if (extension_loaded($class['cdata'])) $status = 2;
									else {
										if ($class['attrs']['type'] == 'minimum') $status = 0;
										else $status = 1;
									}
									$this->addResult('php.extension', array('status' => $status, 'extension' => $extension['cdata']));
								}
							}
							else EventHandler::fireAction($this, 'parseChild');
						}
						catch (Exception $e) {
							// just ignore the exception and show a message in the template lateron
							$this->failed++;
						}
					}
				break;
				// mysql checks
				case 'mysql':
					foreach ($this->block['children'] as $this->child) {
						try {
							if ($this->child['name'] == 'versions') {
								if (WCF::getDB() instanceof MySQLDatabase || WCF::getDB() instanceof MySQLiDatabase) {
									$installedVersion = WCF::getDB()->getVersion();
								}
								$status = 0;
								$hasRecommend = false;
								foreach ($this->child['children'] as $version) {
									if ($version['attrs']['type'] == 'recommend') $hasRecommend = true;
									if (version_compare($installedVersion, $version['cdata']) >= 0) {
										if ($version['attrs']['type'] == 'minimum' && $status < 1) $status = 1;
										else if ($version['attrs']['type'] == 'recommend' && $status < 2) $status = 2;
									}
								}

								// if there is only a minimum, handle it like recommend
								if (!$hasRecommend && $status == 1) $status = 2;
								$this->addResult('mysql.version', array('status' => $status, 'version' => $installedVersion));
							}
							else EventHandler::fireAction($this, 'parseChild');
						}
						catch (Exception $e) {
							// just ignore the exception and show a message in the template lateron
							$this->failed++;
						}
					}
				break;
				// checks defined by third-plugins
				default:
					EventHandler::fireAction($this, 'parseBlock');
			}
		}
		if ($this->overallStatus == 0 || !isset($_POST['send'])) {
			WCF::getTPL()->assign(array(
				'results' => $this->results,
				'overallStatus' => $this->overallStatus,
				'failed' => $this->failed
			));
			WCF::getTPL()->display('packageInstallationShowRequirements');
			exit;
		}
	}

	public function addResult($type, $data) {
		if (!isset($this->results[$type])) $this->results[$type] = array();

		$this->results[$type][] = $data;
		if ($data['status'] < $this->overallStatus) $this->overallStatus = $data['status'];
	}

	public function hasUninstall() {
		// overwritten because we dont save any data
		EventHandler::fireAction($this, 'hasUninstall');

		return false;
	}
}
?>